<?php

function comms_add_user_by_name_if_not_exist($follower) {
  if (variable_get('comms_debuglevel', 0) > 0) {
    drupal_set_message('comms_add_user_if_not_exist()');
  }
  
  if (user_load(array('name' => $follower['name']))) {
    return;
  }

  //add the user if it does not exist
  $account = (object) array();
  $options['name'] = $follower['name'];
  $options['mail'] = $follower['name'] .'@twitter.com';

  if ($follower['twitterUID']) {
    $options['twitterUID'] = $follower['twitterUID'];
  }

  $result = user_save($account, $options);
  if ($result) {
    watchdog('comms', t('Adding new twitter follower, @name.', array('@name' => $result->name)));
  }
  else {
    watchdog('comms', t('Failed adding the twitter follower, @name.', array('@name' => $follower['name'])));
  }
  return $result;
}

//will try to get all new from messages (random tweets) and add them to the
//system
function comms_add_twitter_tweets() {
  if (variable_get('comms_debuglevel', 0) > 0) {
    drupal_set_message('comms_add_twitter_tweets()');
  }

  //for each twitter user
  //get all messages sent through the search api
  //note we should not add the message id here.  If we do we need to rethink
  //the logic above

  //get the list of our groups
  $tusers = array();
  $groups = module_invoke('og', 'all_groups_options');
  while ($name = current($groups)) {
    $account = comms_get_group_twitter_account(key($groups));
    $tusers = array_merge($tusers, array($account['username'] => key($groups)));
    next($groups);
  }

  $messages = comms_get_twitter_tweets('handle_comms_from_twitter');
}//comms_add_twitter_tweets

/**
 * Handles a comms item from twitter.
 *
 * Adds a new comms node and updates the the users comms_twitterfrommsgmaxid.
 *
 * @param $message an array with at least the following keys:
 * array(
 *   'message' => <twitter_message>,
 *   'user' => <twitter_user that has already been added to the system>,
 *   'time' => <timestamp of when the tweet was receved>,
 * );
 *
 * @reutrn TRUE on success else FALSE
 */
function handle_comms_from_twitter($message) {
  $errors = comms_add_comms_from_twitter($message['message'], 
    $message['user'], $message['time']);

  if (count($errors)) {
    watchdog('comms_group', 'handle_comms_from_twitter() did not succeed: '.  
      implode(', ', $errors));
    return FALSE;
  }
  else {
    $user = user_load(array('name' => $message['user']));
    if ($user->comms_twitterfrommsgmaxid < $message['id']) {
      user_save($user,array('comms_twitterfrommsgmaxid' => $message['id']));
    }
  }

  return TRUE;
}

//gets all users that were added from twitter.  Returns an array of twitter
//user names.
function comms_get_twitter_users() {
  if (variable_get('comms_debuglevel', 0) > 0) {
    drupal_set_message('comms_get_twitter_users()');
  }

  $return = NULL;

  if (! ($result = db_query('SELECT uid from {users}'))) {
    watchdog('comms', 
      t('Could not select the twitter users from the database.'));  
    return $return;
  } 

  while ($dbuser = db_fetch_array($result)) {
    $user = user_load(array('uid' => $dbuser['uid']));
    if ($user === FALSE) {
      continue; 
    }

    if ($user->twitterUID) {
      // This is where we can filter out pulling tweets for users below a 
      // certain point value.
      $point_threshold = variable_get('comms_disablepostbelowpoint', 0);
      if ($point_threshold != 0) {
        if (userpoints_get_current_points($user->uid) > $point_threshold) {
          $return[] = $user->name;
        }
      }
      else {
        $return[] = $user->name;
      }
    }
  }//while
  return $return;
}

//gets all the followers for a a group that has a twitter account associated
//with it.  returns an an array of associative arrays in the form:
/*
  Array
  (
    Array
    (
      [name] => <twitter username>,
      [twitterUID] => <twitter user uid>
    )[,...]
  )

 */
function comms_get_twitter_followers($gid) {
  if (variable_get('comms_debuglevel', 0) > 0) {
    drupal_set_message('comms_get_twitter_followers()');
  }

  $return = NULL;

  //if there is no twitter account associated borf!
  $ta = comms_get_group_twitter_account($gid);
  if (is_null($ta)) {
    return $ta;
  }
  
  //there is a situation here were we will only get 100 users from a request in order
  //that they created their accounts.  If we have more then 100 users we would like to
  //get all of them
  $twitter = _comms_twitterAPI_load($ta['username'], $ta['password']);  
  $page = 1;
  while (1) {
    try {
      $messages = $twitter->getFollowers('json', $page);
    }catch (Exception $e) {
      watchdog('comms_group', t('unable to get followers for gid:@gid, %mess',
        array('@gid' => $gid, '%mess' => $e->getMessage())));

      comms_add_twitter_api_call(-1, $gid, 'getFollowers', 1, 0, 
        $e->getMessage()); 

      return $return;
    }
    
    if ($messages->isError()) {
      $error_message_web = scraping_twitter_error_page($messages->getData());
      $error_message_json = json_decode($messages->getData(), 1);

      $message = t("There was a problem trying to get the followers associated with gid:@gid. Error: @errorweb. Error: @errorjson", array(
        '@gid' => $gid,
        '@errorweb' => $error_message_web,
        '@errorjson' => $error_message_json['error']));

      watchdog("comms_group", $message);
      
      comms_add_twitter_api_call(-1, $gid, 'getFollowers', 1, 0, $message);

      return $return;
    }
  else {
      $userct = 0;
      foreach (json_decode($messages->getData(), 1) as $message) {
        $userct++;
        $name = $message['screen_name'];
        $uid = $message['id'];
        $msg = isset($message['status']['text']) ? $message['status']['text'] : NULL;
        $msgid = isset($message['status']['id']) ? $message['status']['id'] : NULL;
        $return[] = array(
          'name' => $name, 
          'twitterUID' => $uid,
          'message' => $msg,
          'message_id' => $msgid
        );
      }//foreach

      if ($userct == 100) {
        $page++;
      }
      else {
        $message = t('Got %count followers for gid: @gid',
          array('%count' => count($return), '@gid' => $gid));
        comms_add_twitter_api_call(-1, $gid, 'getFollowers', 1, 1, $message); 
        return $return ? $return : array();
      }//else
    }//else
  }//while
}//comms_get_twitter_followers

/**
 * Retrives tweets for all twitter users in the system using the Twitter
 * Search API.  Messages will be newer than the most recent message that
 * was pulled for the user.  Users that do not have a maxmsgid will only 
 * retrieve the users first message.
 *
 * @param $callback A call back function that will be passed each tweet.
 * 
 * @todo Ideally we would like to only retrieve messages for new users since 
 * they started following one of our groups, but I am not sure if there is a 
 * way to determine this.
 *
 * @todo determine what the limit is on the number of from: searches we can do
 * at one time and break the search up into multiple requests
 *
 * @todo: maybe we should limit this to a specific time as well ... for 
 * example maybe we only search for tweets newer then today.
 * 
 * @return If $callback was specified TRUE will be returned on success or a 
 * count of the number of errors that were recieved.  If $callback is not
 * specified then an array of messages in the following format will be 
 * retuned:
 * array(
 *   Array (
 *     [user] => <twitter username>,
 *     [message] => <tweet>,
 *     [id] => <twitter message id>,
 *     [created_at] => <time reported by twitter that the tweet was created>,
 *     [time] => strtotime([created_at]),
 *     [twitterUID] => <twitter sender uid>
 *   )[, ...]
 * );
 */
function comms_get_twitter_tweets($callback = null) {
  if (variable_get('comms_debuglevel', 0) > 0) {
    drupal_set_message('comms_get_twitter_tweets()');
  }

  $return = NULL;
  $errorct = 0;

  $tusers = comms_get_twitter_users();

  if (count($tusers) <= 0) {
    return $return;
  }

  $twitter = _comms_twitterSearchAPI_load();  

  // Run a separate twitter search for each user.
  foreach($tusers as $tuser) {
	$tuser_object = user_load(array('name' => $tuser));
    $maxmsgid = $tuser_object->comms_twitterfrommsgmaxid;
    if(!$maxmsgid) {
      $maxmsgid = 0;
    }
    $page=1;
    $break=0;
    //There is a situation where we may have more then 100 tweets for this user
    //since the last check
    while (!$break && $page<5) {
      try {
        $messages = $twitter->getTweetsForUsers(array($tuser), 'json', $maxmsgid, $page);
      }catch (Exception $e) {
        $message =  t("There was a problem trying to get the tweets associated with user: @user. Error Message: @mess", array( 
          '@user' => $tuser, '@mess' => $e->getMessage()));

        watchdog('comms_group', $message);

        comms_add_twitter_api_call(-1, -1, 'twitterSearchAPI::from()', 0, 0,
          $message);

        $break = 1;
        $errorct++;
        continue;
      }
      
      if ($messages->isError()) {

        //needed to get the error message when twitter can't even give a proper error
        $error_message_web = scraping_twitter_error_page($messages->getData());
        $error_message_json = json_decode($messages->getData(), 1);

        $message =  t("There was a problem trying to get the tweets associated with user: @user.Error Message: @errorweb. Error Message: @errorjson.", array(
          '@user' => $tuser,
          '@errorweb' => $error_message_web,
          '@errorjson' => $error_message_json['error']));

        watchdog("comms_group", $message);
        
        comms_add_twitter_api_call(-1, -1, 'twitterSearchAPI::from()', 0, 0, 
          $message);

        $break = 1;
        $errorct++;
        continue;
      }
      else {
        $message_list = json_decode($messages->getData(), 1);
        $message_list = $message_list['results'];
        
        if (is_array($message_list)) {
          $msgct = 0;
          print_r('$message_list for '. $tuser .': '. $message_list);
          foreach ($message_list as $message) {
            $msgct++;
            $text = $message['text'];
            $sender = $message['from_user'];
            $uid = $message['from_user_id'];
            $id = $message['id'];
            //$return[] = array(
            $parsed_message = array(
              'user' => $sender, 
              'message' => $text, 
              'id' => $id,
              'time' => strtotime($message['created_at']),
              'created_at' => $message['created_at'],
              'twitterUID' => $id);
            if ($callback) {
              $callback($parsed_message);
            }
            else {
              $return[] = $parsed_message;
            }

            // If the user did not have a max msg id then we are only
            // going to take their first post.
            // @see http://drupal.org/node/567476 #20.
            if ($maxmsgid == 0) {
              $break =1;
              break;
            }
          }//foreach

          ($msgct >= 100) ? $page++ : $break=1;
           
        }
        else {
          $break=1;
        }

        if (!$callback) {
          $message = t('Got %count tweets for user: @user', array(
            '%count' => count($return), '@user' => $tuser));
        }
        else {
          $message = t('Requested tweets for user: @user', array(
            '@user' => $tuser));
        }
        comms_add_twitter_api_call(-1, -1, 'twitterSearchAPI::from()', 0, 1, 
          $message); 
      }//else
    }//while
  }//foreach

  if ($callback) {
    return $errorct ? $errorct : TRUE;
  }

  $message = t('Got %count tweets for users: @user', array(
    '%count' => count($return), '@user' => implode(', ', $tusers)));
  watchdog("comms_group", $message);

  return $return;
}

//gets all direct messages for a group that has a twitter account associated
//with it.  returns an array of associative arrays in the form:
/*
  Array
  (
    Array
    (
      [user] => <twitter username>,
      [message] => <twitter direct message>,
      [id] => <twitter message id>,
      [twitterUID] => <twitter sender uid>
    )[, ...]
  )
*/
function comms_get_twitter_group_direct_messages($gid) {
  if (variable_get('comms_debuglevel', 0) > 0) {
    drupal_set_message('comms_get_twitter_group_direct_messages()');
  }

  $return = NULL;

  //if there is no twitter account associated borf!
  $ta = comms_get_group_twitter_account($gid);
  if (is_null($ta)) {
    return $ta;
  }
  
  $twitter = _comms_twitterAPI_load($ta['username'], $ta['password']);  

  //get the most recently recorded twitter direct message id 
  $msgid = variable_get('comms_twitterdirectmsgmaxid', 0);

  //this function call returns that last 20 direct messages that were recieved
  //there is a possibility that we have recieved more then 20 direct messages
  //since the last time that we checked, so we should continue to pull messages
  //until we recieve one that is < comms_twitterdirectmsgmaxid  
  $page = 1;
  while (1) {
    try {
      $messages = $twitter->getMessages('json', '', $msgid, $page);
    }catch (Exception $e) {
      watchdog('comms_group', t('unable to get direct messages for gid: @gid, message: @mess', array( 
        array('@gid' => $gid, '@mess' => $e->getMessage()))));

      comms_add_twitter_api_call(-1, $gid, 'getMessages', 1, 0, 
        $e->getMessage()); 

      return $return;
    }
    
    if ($messages->isError()) {
      $error_message_web = scraping_twitter_error_page($messages->getData());
      $error_message_json = json_decode($messages->getData(), 1);

      $message = t("There was a problem trying to get the followers associated with gid:@gid. Error: @errorweb. Error: @errorjson", array(
        '@gid' => $gid,
        '@errorweb' => $error_message_web,
        '@errorjson' => $error_message_json['error']));

      watchdog("comms_group", $message);
      
      comms_add_twitter_api_call(-1, $gid, 'getMessages', 1, 0, $message);

      return $return;
    }
  else {
      $messagect = 0;
      foreach (json_decode($messages->getData(), 1) as $message) {
        $messagect++;
        $text = $message['text'];
          $sender = $message['sender']['screen_name'];
        $uid = $message['sender']['id'];
        $id = $message['id'];
        $return[] = array(
          'user' => $sender, 
          'message' => $text, 
          'id' => $id,
          'twitterUID' => $id);
      }//foreach

      if ($messagect == 20) {
        $page++;
      }
      else {
        $message = t('Got %count tweets for users: @users', array(
          '%count' => count($return), '@users' => $tuser));
        watchdog("comms_group", $message);
        comms_add_twitter_api_call(-1, $gid, 'getMessages', 1, 1, $message); 
        return $return;
      }
    }//else
  }//while
  
}

//Helper function for processing and adding status messages returned
//from comms_get_twitter_followers()
//@todo this needs to have the time added to it.
function comms_add_from_get_followers($followers) {
  if (variable_get('comms_debuglevel', 0) > 0) {
    drupal_set_message('comms_add_from_get_followers()');
  }

  //we don't want to pull in data older then what we already have. With the
  //exception that if we have not pulled anything in with this method before
  //message id's from followers are not ordered
  $max_msg_id = variable_get('comms_twitterfollowermsgmaxid', 0);
  $new_max_msg_id = 0;
  if (is_array($followers)) {
    foreach ($followers as $name => $follower) {

      //some of the followers will not have a status associated with them
      //we will just skip them
      if (!isset($follower['message']) || 
        !isset($follower['message_id']) || 
        !isset($follower['name'])) {
        continue;
      }//if

      if ($follower['message_id'] > $max_msg_id) {
        $errors = comms_add_comms_from_twitter($follower['message'], 
          $follower['name']);
        
        if (count($errors)) {
          watchdog('comms_group', 
            t('comms_cron() did not succeed: %errors',
              array('%errors' => print_r(check_plain($errors), 1))));
        }
  else if ($follower['message_id'] > $new_max_msg_id) {
            $new_max_msg_id = $follower['message_id'];
        }
      }
    }//foreach

    if ($new_max_msg_id > $max_msg_id) {
      variable_set('comms_twitterfollowermsgmaxid', $new_max_msg_id);
    }//if
  }//if  
}

function comms_add_comms_from_twitter($message, $user, $time = NULL, $reset = NULL) {
  if (!isset($message) || !isset($user)) {
    return;
  }

  static $tusers;  
  if (!isset($tusers) || $reset) {
    //get the list of our groups
    $tusers = array();
    $groups = module_invoke('og', 'all_groups_options');
    while ($name = current($groups)) {
      $account = comms_get_group_twitter_account(key($groups));
      $tusers = array_merge($tusers, array($account['username'] => key($groups)));
      next($groups);
    }
  }

  $values = array();
  // @todo use the drupal time function for this.
  $values['body'] = t('pulled from twitter at %date.', array(
    '%date' => date('D, m/d/Y - H:i:s T')));
  
  //there can be encoded html characters in the message, we handle 
  //them here.
  $message = htmlspecialchars_decode($message);

  //see simplefeed_item.module for an example of this
  //we want to make sure here that this is nt a tweet meant for a 
  //group that we are not trancking
  //then if this is a message directed at another user that is not
  //one of the groups we care about we ignore it
  //http://help.twitter.com/index.php?pg=kb.page&id=75  
  if (preg_match('/^@(\w+)\s(.*)$/', $message, $matches)) {
    if (array_key_exists($matches[1], $tusers)) {
      $values['sms_message'] = $matches[2];
      $values['body'] = htmlspecialchars_decode($message) ."\n".
        $values['body'];
      $values['to'] = $matches[1];
    }
  else {
      return;
    }
    

  }//if
  else {
    $values['body'] = $message ."\n". $values['body'];
    $values['sms_message'] = $message;
  }
  
  $node->type = 'comms';
  
  //for the title we want the first 8 words
  $values['title'] = split(' ', $values['sms_message'], 8);
  if (count($values['title']) == 8) {
    array_pop($values['title']);
  }
  $values['title'] = implode(' ', $values['title']);

  //we want to add the group membership if there is one  
  if (!empty($values['to'])) {
    $values['og_groups'] = array($tusers[$values['to']] => $tusers[$values['to']]);
  }

  $values['name'] = $user;
  $time = $time ? $time+variable_get('date_default_timezone', 0) : time();
  $values['date'] = date('Y-m-d H:i:s', $time);


  drupal_execute('comms_node_form', $values, $node);
  $errors = form_get_errors();

  return $errors;
}

