<?php
// Include some awesome logger.
// http://www.indelible.org/php/Log/guide.html
require_once "Log.php";

// Load the config.
require_once dirname(__FILE__) .'/tapatio_poll_twitter.inc';

global $base_url, $base_path;
$base_path = 'skdfjsdkljfsldkj';

class Tapatio {
  private $config;
  private $logger;
  const MAX_NUM_TWEET_PAGES = 5;

  // @todo This class should have its own logger.
  public function __construct() {
    global $base_path, $base_url;
    $this->config = new Config;
    $this->logger = Log::factory(
      $this->config->logger_conf['type'], '', 
      'threadedTwitterUserFeeder', 
      $this->config->logger_options[$this->config->logger_conf['type']], 
      //$this->config->logger_conf['level']);
      PEAR_LOG_DEBUG);

    $_SERVER['HTTP_HOST'] = $this->config->hostname;
    $_SERVER['REMOTE_ADDR'] = '127.0.0.1';
    $_SERVER['REQUEST_METHOD'] = 'GET';
    $_SERVER['REQUEST_URI'] = $_SERVER['SCRIPT_NAME'] = $_SERVER['PHP_SELF'];
    $base_url = 'http://'. $_SERVER['HTTP_HOST'];
    $base_path = $this->config->drupal_path;
    chdir($base_path);

    //require_once $this->config->drupal_path . '/includes/bootstrap.inc';
    require_once 'includes/bootstrap.inc';

    // We can access the database api after this.
    //drupal_bootstrap(DRUPAL_BOOTSTRAP_DATABASE);

    // But we can not access functions that leverage both the database API
    // and the cache like variable_get untill after this.
    //drupal_bootstrap(DRUPAL_BOOTSTRAP_LATE_PAGE_CACHE);

    // We can not use the full api until however;
    // @todo : I think drupal overwrites the default log handlers to use 
    // watchdog().
    ob_start();
    drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
    ob_end_clean();
    restore_error_handler();

    $this->twitter = _comms_twitterSearchAPI_load();
    if (is_null($this->twitter)) {
      $this->logger->crit('Unable to initialize a twitter object.');
      exit;
    }
  }

  /**
   * Get a list of tapatio groups.
   *
   * @return an array of hashes describing the tapatio groups with the 
   * following structure:
   * array(
   *   <group_name> = array(
   *     'twitter_user_name' = <twitter_user_name>,
   *     'twitter_password' = <twitter_password>
   *   ),
   *   ...
   * );
   */
  public function getGroups() {
    $sql = 'SELECT gta.username username, gta.password password, '.
      'node.title title '.
      'from {group_twitter_account} AS gta '.
      'LEFT JOIN node on (gta.nid = node.nid)';
    $result = db_query($sql);
    while ($gta = db_fetch_object($result)) {
      $return[$gta->title]['twitter_user_name'] = $gta->username;
      $return[$gta->title]['twitter_password'] = $gta->username;
    }

    return $return;
  }

  /**
   * Get a list of twitter followers for a group.
   *
   * This function will not return users
   *
   * @return an array of followers with the following structure:
   * array(
   *    <twitter_user_name> => array(
   *      'twitterUID' => <twitterUID>,
   *      'comms_twitterfrommsgmaxid' => <comms_twitterfrommsgmaxid>
   *    ),
   *    ...
   * );
   */
  public function getFollowers() {
    $sql = 'SELECT * from {users} where mail like "%twitter.com"';
    $result = db_query($sql);
    $return = array();
    while ($user_row = db_fetch_object($result)) {
      $user = user_load(array('uid' => $user_row->uid));
      $return[$user->name] = array(
        'twitterUID' => $user->twitterUID,
        'comms_twitterfrommsgmaxid' => $user->comms_twitterfrommsgmaxid
      );
    }
    return $return;
  }

  /**
   * Report if a user pulled into tapatio is "banned".  
   * That is if 
   * "Disable pulling in tweets for users with points below" has been set
   * and the users points is below this threshold.
   *
   * @arg $username The username as it will be in the users table.
   *
   * @return TRUE if the user is banned else FALSE
   */
  public function isUserBaned($username) {
    // Check to see if "Disable pulling in tweets for users with points below"
    // is enabled.
    $disabled_with_points = variable_get('comms_disablepostbelowpoint', 0);
    if (!$disabled_with_points) {
      return FALSE;
    }

    $user = user_load(array('name' => $username));
    $sql = 'SELECT points from {userpoints} where uid=%d';
    $result = db_query($sql, $user->uid);
    $points = db_fetch_object($result);
    if ($points->points <= $disabled_with_points) {
      return TRUE;
    } 
    return FALSE;
  }

  /**
   * Returns a list of tweets for a user.
   *
   * @arg the twitter user stored in drupal to get the tweets for.
   *
   * @result a list of tweets for a user in the following format:
   * array (
   *   array(
   *     'user' => <sender>,
   *     'message' => <tweet>,
   *     'id' => <tweet_message_id>,
   *     'twitterUID' => <sender_twitter_id>
   *   ),
   *   ...
   * );
   * or FALSE on error.
   */
  public function getNewTweetsForUser($username) {
    $user = user_load(array('name' => $username));
    if (!$user->uid) {
      $this->logger->err('getNewTweetsForUser() can not get tweets for non existent tapatio user '. $username);
      $this->logger->debug(print_r($user, 1));
      return FALSE;
    }

    $return = array();
    $break = 0;
    $page = 0;
    $maxmsgid = $user->comms_twitterfrommsgmaxid ? 
      $user->comms_twitterfrommsgmaxid : 0;
    $this->logger->debug('Should not get any tweets > '. $maxmsgid);
    while (!$break && $page < self::MAX_NUM_TWEET_PAGES) {
      try {
        $messages = $this->twitter->getTweetsForUsers(
          array($user->name), 'json', $maxmsgid, $page);
      }
      catch(Exception $e) {
        $this->logger->err("getNewTweetsForUser() There was a problem trying to get the tweets associated with $user->name. Error Message: ". $e->getMessage());
        comms_add_twitter_api_call(-1, -1, 'twitterSearchAPI::from()', 0, 0,  
          $message);                                                          
      
        $break = 1;                                                           
        continue;
      }           
      if ($messages->isError()) {                                             
        // Needed to get the error message when twitter can't even give a 
        // proper error.
        $error_message_web = 
          scraping_twitter_error_page($messages->getData());  
        $error_message_json = json_decode($messages->getData(), 1);

        $this->logger->err("There was a problem trying to get the tweets associated with $user->name. Error Message: $error_message_web. Error Message: ". $error_message_json['error']);

        comms_add_twitter_api_call(-1, -1, 'twitterSearchAPI::from()', 0, 0,
          $message);

        $break = 1;
        continue;
      }
      else {
        $message_list = json_decode($messages->getData(), 1);
        $message_list = $message_list['results'];

        if (is_array($message_list)) {
          $msgct = 0;
          foreach ($message_list as $message) {
            $msgct++;
            $text = $message['text'];
            $id = $message['id'];
            $sender = $message['from_user'];
            $return[] = array(
              'user' => $sender,
              'message' => $text,
              'id' => $id,
              'time' => strtotime($message['created_at']),
              'created_at' => $message['created_at'],
              'uid' => $user->uid,
              'twitterUID' => $id);
          }//foreach

          ($msgct >= 100) ? $page++ : $break=1;
        }
        else {
          $break=1;
        }

        $message = t('Got %count tweets for user: %user', array(
          '%count' => count($return), '%user' => $user->name));
        comms_add_twitter_api_call(-1, -1, 'twitterSearchAPI::from()', 0, 1,
          $message);
        $this->logger->info($message);
      }//else
    }

    return $return;
  }

  /**
   * Will get all new tweets for a user unless they are banned, updating the 
   * users twitter message id as needed.
   *
   * @arg $username tha tapatio/twitter username to get the tweets for.
   *
   * @return FALSE on FAILURE else the number of twets added.
   */
  public function handleTweetsForUser($username) {
    if ($this->isUserBaned($username)) {
      return FALSE;
    }

    $messages = $this->getNewTweetsForUser($username);
    if (!$messages) {
      return FALSE;
    }

    $groups = $this->getGroupsByOG();
    if (empty($groups)) {
      $this->logger->warn('handleTweetsForUser() No tweets will be associated with any groups because we were unable to get a list of groups fromthe tapatio system.');
    }

    $return = 0;
    $maxmsgid = 0;
    foreach ($messages as $message) {
      $this->logger->debug("handleTweetsForUser() adding comms via node_save():\n".
        "\tmessage: ". $message['message'] ."\n".
        "\tuser: ". $message['user'] ."\n".
        "\tuid: ". $message['uid'] ."\n".
        "\tdate string: ". $message['created_at'] ."\n".
        "\ttime: ". $message['time']);

      $node = new StdClass();
      $node->body = 'pulled from twitter at '. 
        date('D, m/d/Y - H:i:s T'); 

      // We don't care about this message if it is not referencing one of
      // our groups.
      if (
        preg_match('/^@(\w+)\s(.*)$/', $message['message'], $matches) && 
        !empty($groups)
      ) {
        if ($gid = array_search($matches[1], $groups)) {
          $new_message = htmlspecialchars_decode($matches[2]);
          $node->sms_message = $new_message;
          $node->og_groups[] = $gid;
          $nody->body = htmlspecialchars_decode($message['message']) ."\n".
            $node->body;

          //for the title we want the first 8 words
          $title = split(' ', $new_message, 8);
          if (count($title) == 8) {
            array_pop($title);
          }
          $node->title = implode(' ', $title);
        }
        else {
          $this->logger->info("Skipping message for $username as it refers to a tapatio group we are not tracking.");
          $this->logger->debug(implode("\n\t", $message));
          continue;
        }
      }
      else {
        $message['message'] = htmlspecialchars_decode($message['message']);
        $node->body = $message['message'] ."\n". $node->body;

        //for the title we want the first 8 words
        $title = split(' ', $message['message'], 8);
        if (count($title) == 8) {
          array_pop($title);
        }
        $node->title = implode(' ', $title);
        $node->sms_message = $message['message'];
      }

      $node->type = 'comms';
      $node->name = $message['user'];
      $node->uid = $message['uid'];
      $node->status = 1;

      if ($message['id'] > $maxmsgid) {
        $maxmsgid = $message['id'];
      }
      // This is fucked, but it seems to be the only way to account for the time zone.
      $timezone = variable_get('date_default_timezone', 0);
      $node->created = $message['time'] + $timezone;
      node_save($node);
      $return++;
    }

    $user = user_load(array('name' => $username));
    if (!$user->uid) {
      $this->logger->err('handleTweetsForUser() can not save '. $username);
    }
    if ($user->comms_twitterfrommsgmaxid < $maxmsgid) {
      $this->logger->debug('Updating '. $username .'\'s comms_twitterfrommsgmaxid setting to '. $maxmsgid);
      user_save($user, array('comms_twitterfrommsgmaxid' => $maxmsgid));
    }

    return $return;
  }

  public function getGroupsByOG() {
    $return = array();
    // Get the group type for tapatio groups.
    $group_type = variable_get('comms_group_node_type', NULL);
    if (is_null($group_type)) {
      $this->logger->warn('getGroups(): Tapatio system is not configured.  Please set the comms_group_node_type variable.');
      return $return;
    }

    $sql = 'SELECT nid,username from {group_twitter_account}';
    $result = db_query($sql);
    while ($node = db_fetch_object($result)) {
      $return[$node->nid] = $node->username;
    }
    return $return;
  }

  /**
   * Will get the last group that used the twitter API.
   *
   * @return FALSE on FAILURE else an array in the following format:
   * array(
   *   'username' => <group_name>,
   *   'timestamp' => <time the last twitter api call was made>
   * );
   */
  public function getLastGroupThatUsedAPI() {
    $sql = 'SELECT username, MAX( call_time ) call_time '.
      'FROM group_twitter_account '.
      'LEFT JOIN twitter_api_calls ON '.
        '(group_twitter_account.nid = twitter_api_calls.gid AND gid >0 ) '.
      'GROUP BY username '.
      'ORDER BY call_time '.
      'LIMIT 1';
    $result = db_query($sql);
    $row = db_fetch_object($result);
    return array(
      'username' => $row->username, 
      'timestamp' => strtotime($row->call_time)
    );
  }

  /**
   * Gets the status for a group, and adds new followers.
   *
   * @arg $group the twitter username acting as a tapatio group.
   *
   * @returns FALSE on error else an array of new users in the following 
   * format: array(
   *   <username>,
   *   ...
   * );
   */
  public function handleStatusForGroup($group) {
    // Verify this is a group we are tracking.
    $groups = $this->getGroupsByOG();
    if (!array_search($group, $groups)) {
      $this->logger->err('handleStatusForGroup() recieved an invalid group '. $group);
      return FALSE;
    } 

    $sql = 'SELECT nid FROM {group_twitter_account} WHERE username="%s"';
    $result = db_query($sql, $group);
    $row = db_fetch_object($result);
    if (empty($row) || empty($row->nid)) {
      $this->logger->err('Not able to get the gid for group '. $group);
      return FALSE;
    }

    $followers = comms_get_twitter_followers($row->nid);
    if (is_null($followers)) {
      $this->logger->err('Not able to get followers for goup '. $group .'. Check the TAPI logs.');
      return FALSE;
    }
    if (empty($followers)) {
      $this->logger->err('No followers were found for goup '. $group);
      return FALSE;
    }

    $return = array();
    foreach ($followers as $follower) {
      if ($this->addFollower($follower['name'], $follower['twitterUID'])) {
        $return[] = $follower['name'];
      }
    }

    $this->logger->debug('Handled '. count($followers) .' followers for '. $group);
    return $return;
  }

  /**
   * Add a follower to the tapatio system.  
   * This consists of adding a new drupal user.
   *
   * @arg $follower the twitter username of the follower to add.
   *
   * @return FALSE if already exists else the drupal user object;
   */
  public function addFollower($follower, $twitterUID = NULL) {
    if ($user = user_load(array('name' => $follower))) {
      return FALSE;
    }

    $user = new stdClass;
    return user_save($user, array(
      'name' => $follower,
      'mail' => $follower .'@twitter.com',
      'twitterUID' => $twitterUID 
    ));
  }

  /**
   * Add a comms entry to the tapatio system.
   *
   * @arg 
   */
  public function addComms() {}

  /**
   * Get the last Twitter API (TAPI) Call made for a group.
   *
   * @arg $group the twitter group.
   *
   * @return a hash of information about the last TAPI call made for the group
   * with the following structure:
   * array(
   * );
   */
  public function getLastTAPICallForGroup($group) {}

  /**
   * Add a TAPI call for a group.
   *
   * @arg $tapi_call a hash with the following structure:
   * array(
   * );
   */
  public function addTAPICallForGroup($tapi_call) {}
}

//$tapatio = new Tapatio();
//$group = $tapatio->getLastGroupThatUsedAPI();
//$new_users = $tapatio->handleStatusForGroup($group['username']);
//$group = $tapatio->getLastGroupThatUsedAPI();
//print_r($group);
//$messages = $tapatio->getNewTweetsForUser('postmodern_mod3');
//$messages = $tapatio->handleTweetsForUser('postmodern_mod3');
//print_r($messages);
/*
print_r($tapatio->getGroups());
print_r($tapatio->getFollowers());
print date('D, m/d/Y - h:i', strtotime('Fri, 18 Sep 2009 09:21:04 +0000')) ."\n";
print date('D, m/d/Y - h:i', 1253265664) ."\n";
 */
