<?php
// Include multithreaded lib.
// http://code.google.com/p/phpmultithreadeddaemon/
require_once dirname(__FILE__) .'/class.MTDaemon.php';

// Include some awesome logger.
// http://www.indelible.org/php/Log/guide.html
require_once "Log.php";

// Load the config.
require_once dirname(__FILE__) .'/tapatio_poll_twitter.inc';

// Load the tapatio library.
require_once dirname(__FILE__) .'/class.Tapatio.php';

// Queue feeder: semi regularly pull in the groups we are managing and look
// for new followers, add the followers if necesary.  If disabling pulling
// feeds below a certain threshold is set check if users are below threshold.
// If they are remove them from the queue.
// The parent feeder will dispatch a set of threads to pull the followers for 
// each group.  Followers will be shared through an ipc stack.

// Queue Consumer.  Get all new tweets for the user through the twitter search 
// api greater then that last msgid for the user.  Post the new message, update
// the users max msg id, push the user back on the bottom of the stack.
class threadedTwitterUserFeeder extends MTDaemon {
  public $stack_name = 'stack';
  private $config;
  private $groups;
  private $tapatio;
  private $consumer;

  // Twitter only allows 150 requests per hour through the API.
  // 60*60/150 = 24
  const MIN_WAIT_SECS = 24;

  // @todo This class should have its own logger.
  public function __construct($threads = null, $consumer, $idlesleeptime = null) {
    $this->config = new Config;
    $logger = Log::factory(
      $this->config->logger_conf['type'], '', 
      'threadedTwitterUserFeeder', 
      $this->config->logger_options[$this->config->logger_conf['type']], 
      //$this->config->logger_conf['level']);
      PEAR_LOG_DEBUG);

    parent::__construct($threads, $logger, $idlesleeptime);
    $this->consumer = $consumer;

    $this->tapatio = new Tapatio();
    $followers = $this->tapatio->getFollowers();
    $this->consumer->updateUserList(array_keys($followers));

    // Get the initial list of groups.
    $this->groups = $this->tapatio->getGroups;
  }

  public function getNext ($slot) {
    // Get last group that used the API 
    $group = $this->tapatio->getLastGroupThatUsedAPI();
    $this->logo->debug(
      "\$slot:$slot Last group to use the API: ". print_r($group, 1));
    return $group;
  }

  public function run($group, $slot) {
    $sleep_time = time() - $group['timestamp'];
    // Do some sanity checks on the time here.
    if ($sleep_time > 60*60) {
      $this->logger->warn('The '. $group['username'] .' has not used the twitter api in over an hour.');
    }

    // If it is not time to poll the group sleep till it is.
    if ($sleep_time < threadedTwitterUserFeeder::MIN_WAIT_SECS) {
      $this->logger->debug('Sleeping for '. $sleep_time .' before getting status for '. $group['username']);
      sleep($sleep_time);
    }

    // Have tapatio handle the groups status and give us a list of the new
    // usernames that are following it.
    $new_users = $tapatio->handleStatusForGroup($group['username']);
    $this->consumer->updateUserList($new_users);
  }
}

require_once dirname(__FILE__) .'/class.ThreadedTwitterUserTweetFinder.php';
$consumer = new threadedTwitterUserTweetFinder(5);
$producer = new threadedTwitterUserFeeder(5, $consumer);
$producer->handle();
$consumer->handle();
