<?php
// Include multithreaded lib.
// http://code.google.com/p/phpmultithreadeddaemon/
require_once dirname(__FILE__) .'/class.MTDaemon.php';

// Include some awesome logger.
// http://www.indelible.org/php/Log/guide.html
require_once "Log.php";

// Load the config.
require_once dirname(__FILE__) .'/tapatio_poll_twitter.inc';

// Load the tapatio library.
require_once dirname(__FILE__) .'/class.Tapatio.php';

// Queue feeder: semi regularly pull in the groups we are managing and look
// for new followers, add the followers if necesary.  If disabling pulling
// feeds below a certain threshold is set check if users are below threshold.
// If they are remove them from the queue.
// The parent feeder will dispatch a set of threads to pull the followers for 
// each group.  Followers will be shared through an ipc stack.

// Queue Consumer.  Get all new tweets for the user through the twitter search 
// api greater then that last msgid for the user.  Post the new message, update
// the users max msg id, push the user back on the bottom of the stack.
class threadedTwitterUserTweetFinder extends MTDaemon {
  protected $stack_name = 'stack';
  private $config;
  private $tapatio;
  protected $list_name = 'users';

  // @todo This class should have its own logger.
  public function __construct($threads = null, $idlesleeptime = null) {
    $this->config = new Config;
    $logger = Log::factory(
      $this->config->logger_conf['type'], '', 
      'threadedTwitterUserTweetFinder', 
      $this->config->logger_options[$this->config->logger_conf['type']], 
      $this->config->logger_conf['level']);
    parent::__construct($threads, $logger, $idlesleeptime);

    // Initialize the shared memory variables $stack_name and $list_name.
    if ($lock) $this->lock();
    $vars = shm_get_var($this->shared_data, self::_INDEX_DATA);
    $vars[$this->stack_name] = array();
    $vars[$this->list_name] = array();
    $res = shm_put_var($this->shared_data, self::_INDEX_DATA, $vars);
    if ($lock) $this->unlock();
    if (!$res) {
      $this->logo->err('__construct(): Unable to initialize shared data.');
    }

    $this->tapatio = new Tapatio();
  }

  public function getNext ($slot) {
    $user = $this->shiftStack($this->stack_name);
    $this->logo->debug("\$slot:$slot shifted user:$user off the stack");
    return $user;
  }

  public function run($user, $slot) {
    $this->logo->info("\$slot:$slot getting updates for user $user");
    $tapatio->handleTweetsForUser($user);
    // We push the user back on the stack because they could be unbanned 
    // later.
    $this->pushStack($this->stack_name, $user);
  }

  /*
   * Get an element from the begining of the shared stack.
   *
   * Return null if the stack is empty exist.
   * @todo Throw an exception if the element is not an array.
   */
  public function shiftStack($name, $lock = false)
  {
    if ($lock) $this->lock();
    $vars = shm_get_var($this->shared_data, self::_INDEX_DATA);
    $array = $vars[$name];
    $value = array_shift($array);
    $vars[$name] = $array;
    $res = shm_put_var($this->shared_data, self::_INDEX_DATA, $vars);
    if ($lock) $this->unlock();
    if (!$res) {
      $this->logo->err('shiftStack(): unable to save stack after shifting.');
    }
    return $value;
  }

  /*
   * Push an element onto the end of the stack.
   * 
   * Remark : the var should be serialized.
   */
  public function pushStack($name, $value, $lock = false)
  {
    if ($lock) $this->lock();
    $vars = shm_get_var($this->shared_data, self::_INDEX_DATA);
    array_push($vars[$name], $value);
    $res = shm_put_var($this->shared_data, self::_INDEX_DATA, $vars);
    if ($lock) $this->unlock();
    return $res;
  }

  public function updateUserList($newUsers) {
    if ($lock) $this->lock();
    $vars = shm_get_var($this->shared_data, self::_INDEX_DATA);
    if ($lock) $this->unlock();
    $this->logo->debug('updateUserList() $vars: '. print_r($vars, 1));

    $users = $vars[$this->list_name];
    $changed = 0;

    foreach ($newUsers as $user) {
      if (!in_array($user, $users)) {
        $changed++;
        $this->pushStack($this->stack_name, $user);
        $users[] = $user;
      }
    }

    // @todo what if users are removed?  Should we care?
    if ($changed) {
      if ($lock) $this->lock();
      $vars = shm_get_var($this->shared_data, self::_INDEX_DATA);
      $vars[$this->list_name] = $users;
      $res = shm_put_var($this->shared_data, self::_INDEX_DATA, $vars);
      if ($lock) $this->unlock();
      if (!$res) {
        $this->logo->err('updateUserList(): Unable to update user list.');
      }
    }
    $this->logo->debug('updateUserList() $vars: '. print_r($vars, 1));
  }
}

//$finder = new threadedTwitterUserTweetFinder(5);
